import { Orientation } from '../sash/sash.js';
import { Event } from '../../../common/event.js';
import { Disposable } from '../../../common/lifecycle.js';
import { IView } from './splitview.js';

interface IPaneOptions {
    minimumBodySize?: number;
    maximumBodySize?: number;
    expanded?: boolean;
    orientation?: Orientation;
    title: string;
    titleDescription?: string;
}
interface IPaneStyles {
    readonly dropBackground: string | undefined;
    readonly headerForeground: string | undefined;
    readonly headerBackground: string | undefined;
    readonly headerBorder: string | undefined;
    readonly leftBorder: string | undefined;
}
/**
 * A Pane is a structured SplitView view.
 *
 * WARNING: You must call `render()` after you construct it.
 * It can't be done automatically at the end of the ctor
 * because of the order of property initialization in TypeScript.
 * Subclasses wouldn't be able to set own properties
 * before the `render()` call, thus forbidding their use.
 */
declare abstract class Pane extends Disposable implements IView {
    private static readonly HEADER_SIZE;
    readonly element: HTMLElement;
    private header;
    private body;
    protected _expanded: boolean;
    protected _orientation: Orientation;
    private expandedSize;
    private _headerVisible;
    private _collapsible;
    private _bodyRendered;
    private _minimumBodySize;
    private _maximumBodySize;
    private _ariaHeaderLabel;
    private styles;
    private animationTimer;
    private readonly _onDidChange;
    readonly onDidChange: Event<number | undefined>;
    private readonly _onDidChangeExpansionState;
    readonly onDidChangeExpansionState: Event<boolean>;
    get ariaHeaderLabel(): string;
    set ariaHeaderLabel(newLabel: string);
    get draggableElement(): HTMLElement;
    get dropTargetElement(): HTMLElement;
    get dropBackground(): string | undefined;
    get minimumBodySize(): number;
    set minimumBodySize(size: number);
    get maximumBodySize(): number;
    set maximumBodySize(size: number);
    private get headerSize();
    get minimumSize(): number;
    get maximumSize(): number;
    orthogonalSize: number;
    constructor(options: IPaneOptions);
    isExpanded(): boolean;
    setExpanded(expanded: boolean): boolean;
    get headerVisible(): boolean;
    set headerVisible(visible: boolean);
    get collapsible(): boolean;
    set collapsible(collapsible: boolean);
    get orientation(): Orientation;
    set orientation(orientation: Orientation);
    render(): void;
    layout(size: number): void;
    style(styles: IPaneStyles): void;
    protected updateHeader(): void;
    protected abstract renderHeader(container: HTMLElement): void;
    protected abstract renderBody(container: HTMLElement): void;
    protected abstract layoutBody(height: number, width: number): void;
}
interface IPaneDndController {
    canDrag(pane: Pane): boolean;
    canDrop(pane: Pane, overPane: Pane): boolean;
}
interface IPaneViewOptions {
    dnd?: IPaneDndController;
    orientation?: Orientation;
}

export { type IPaneDndController, type IPaneOptions, type IPaneStyles, type IPaneViewOptions, Pane };
